@@ -1,6 +1,6 @@
 #include <petsc/private/pcmgimpl.h> /*I "petscksp.h" I*/
 
-/*@C
+/*@
   PCMGResidualDefault - Default routine to calculate the residual.
 
   Collective
@@ -24,7 +24,7 @@ PetscErrorCode PCMGResidualDefault(Mat mat, Vec b, Vec x, Vec r)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   PCMGResidualTransposeDefault - Default routine to calculate the residual of the transposed linear system
 
   Collective
@@ -49,7 +49,7 @@ PetscErrorCode PCMGResidualTransposeDefault(Mat mat, Vec b, Vec x, Vec r)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   PCMGMatResidualDefault - Default routine to calculate the residual.
 
   Collective
@@ -69,12 +69,12 @@ PetscErrorCode PCMGResidualTransposeDefault(Mat mat, Vec b, Vec x, Vec r)
 PetscErrorCode PCMGMatResidualDefault(Mat mat, Mat b, Mat x, Mat r)
 {
   PetscFunctionBegin;
-  PetscCall(MatMatMult(mat, x, MAT_REUSE_MATRIX, PETSC_DEFAULT, &r));
+  PetscCall(MatMatMult(mat, x, MAT_REUSE_MATRIX, PETSC_CURRENT, &r));
   PetscCall(MatAYPX(r, -1.0, b, UNKNOWN_NONZERO_PATTERN));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   PCMGMatResidualTransposeDefault - Default routine to calculate the residual of the transposed linear system
 
   Collective
@@ -94,7 +94,7 @@ PetscErrorCode PCMGMatResidualDefault(Mat mat, Mat b, Mat x, Mat r)
 PetscErrorCode PCMGMatResidualTransposeDefault(Mat mat, Mat b, Mat x, Mat r)
 {
   PetscFunctionBegin;
-  PetscCall(MatTransposeMatMult(mat, x, MAT_REUSE_MATRIX, PETSC_DEFAULT, &r));
+  PetscCall(MatTransposeMatMult(mat, x, MAT_REUSE_MATRIX, PETSC_CURRENT, &r));
   PetscCall(MatAYPX(r, -1.0, b, UNKNOWN_NONZERO_PATTERN));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -536,7 +536,7 @@ PetscErrorCode PCMGGetInjection(PC pc, PetscInt l, Mat *mat)
 
   Level: advanced
 
-.seealso: [](ch_ksp), `PCMG`, ``PCMGGetSmootherUp()`, `PCMGGetSmootherDown()`, `PCMGGetCoarseSolve()`
+.seealso: [](ch_ksp), `PCMG`, `PCMGGetSmootherUp()`, `PCMGGetSmootherDown()`, `PCMGGetCoarseSolve()`
 @*/
 PetscErrorCode PCMGGetSmoother(PC pc, PetscInt l, KSP *ksp)
 {
