@@ -10,7 +10,7 @@ int main(int argc, char **args)
   PetscViewer view;
 
   PetscFunctionBeginUser;
-  PetscCall(PetscInitialize(&argc, &args, (char *)0, help));
+  PetscCall(PetscInitialize(&argc, &args, NULL, help));
   PetscCall(MatCreateSeqAIJ(PETSC_COMM_WORLD, m, n, 20, 0, &A));
 
   for (i = 0; i < n; i++) values[i] = (PetscReal)i;
