@@ -69,6 +69,38 @@ static PetscErrorCode TaoSetUpEW_Private(Tao tao)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
+/*@
+  TaoParametersInitialize - Sets all the parameters in `tao` to their default value (when `TaoCreate()` was called) if they
+  currently contain default values. Default values are the parameter values when the object's type is set.
+
+  Collective
+
+  Input Parameter:
+. tao - the `Tao` object
+
+  Level: developer
+
+  Developer Note:
+  This is called by all the `TaoCreate_XXX()` routines.
+
+.seealso: [](ch_snes), `Tao`, `TaoSolve()`, `TaoDestroy()`,
+          `PetscObjectParameterSetDefault()`
+@*/
+PetscErrorCode TaoParametersInitialize(Tao tao)
+{
+  PetscObjectParameterSetDefault(tao, max_it, 10000);
+  PetscObjectParameterSetDefault(tao, max_funcs, PETSC_UNLIMITED);
+  PetscObjectParameterSetDefault(tao, gatol, PetscDefined(USE_REAL_SINGLE) ? 1e-5 : 1e-8);
+  PetscObjectParameterSetDefault(tao, grtol, PetscDefined(USE_REAL_SINGLE) ? 1e-5 : 1e-8);
+  PetscObjectParameterSetDefault(tao, crtol, PetscDefined(USE_REAL_SINGLE) ? 1e-5 : 1e-8);
+  PetscObjectParameterSetDefault(tao, catol, PetscDefined(USE_REAL_SINGLE) ? 1e-5 : 1e-8);
+  PetscObjectParameterSetDefault(tao, gttol, 0.0);
+  PetscObjectParameterSetDefault(tao, steptol, 0.0);
+  PetscObjectParameterSetDefault(tao, fmin, PETSC_NINFINITY);
+  PetscObjectParameterSetDefault(tao, trust0, PETSC_INFINITY);
+  return PETSC_SUCCESS;
+}
+
 /*@
   TaoCreate - Creates a Tao solver
 
@@ -85,7 +117,7 @@ static PetscErrorCode TaoSetUpEW_Private(Tao tao)
 
   Level: beginner
 
-.seealso: [](ch_tao), `Tao`, `TaoSolve()`, `TaoDestroy()`, `TAOSetFromOptions()`, `TAOSetType()`
+.seealso: [](ch_tao), `Tao`, `TaoSolve()`, `TaoDestroy()`, `TaoSetFromOptions()`, `TaoSetType()`
 @*/
 PetscErrorCode TaoCreate(MPI_Comm comm, Tao *newtao)
 {
@@ -95,31 +127,11 @@ PetscErrorCode TaoCreate(MPI_Comm comm, Tao *newtao)
   PetscAssertPointer(newtao, 2);
   PetscCall(TaoInitializePackage());
   PetscCall(TaoLineSearchInitializePackage());
-  PetscCall(PetscHeaderCreate(tao, TAO_CLASSID, "Tao", "Optimization solver", "Tao", comm, TaoDestroy, TaoView));
 
-  /* Set non-NULL defaults */
+  PetscCall(PetscHeaderCreate(tao, TAO_CLASSID, "Tao", "Optimization solver", "Tao", comm, TaoDestroy, TaoView));
   tao->ops->convergencetest = TaoDefaultConvergenceTest;
 
-  tao->max_it    = 10000;
-  tao->max_funcs = -1;
-#if defined(PETSC_USE_REAL_SINGLE)
-  tao->gatol = 1e-5;
-  tao->grtol = 1e-5;
-  tao->crtol = 1e-5;
-  tao->catol = 1e-5;
-#else
-  tao->gatol = 1e-8;
-  tao->grtol = 1e-8;
-  tao->crtol = 1e-8;
-  tao->catol = 1e-8;
-#endif
-  tao->gttol   = 0.0;
-  tao->steptol = 0.0;
-  tao->trust0  = PETSC_INFINITY;
-  tao->fmin    = PETSC_NINFINITY;
-
   tao->hist_reset = PETSC_TRUE;
-
   PetscCall(TaoResetStatistics(tao));
   *newtao = tao;
   PetscFunctionReturn(PETSC_SUCCESS);
@@ -215,7 +227,7 @@ PetscErrorCode TaoSetUp(Tao tao)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TaoDestroy - Destroys the `Tao` context that was created with `TaoCreate()`
 
   Collective
@@ -370,6 +382,7 @@ PetscErrorCode TaoSetFromOptions(Tao tao)
   PetscViewer monviewer;
   PetscBool   flg, found;
   MPI_Comm    comm;
+  PetscReal   catol, crtol, gatol, grtol, gttol;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(tao, TAO_CLASSID, 1);
@@ -394,26 +407,30 @@ PetscErrorCode TaoSetFromOptions(Tao tao)
     if (!prefix) PetscCall(TaoLineSearchSetOptionsPrefix(tao->linesearch, ((PetscObject)tao)->prefix));
   }
 
-  PetscCall(PetscOptionsReal("-tao_catol", "Stop if constraints violations within", "TaoSetConstraintTolerances", tao->catol, &tao->catol, &flg));
-  if (flg) tao->catol_changed = PETSC_TRUE;
-  PetscCall(PetscOptionsReal("-tao_crtol", "Stop if relative constraint violations within", "TaoSetConstraintTolerances", tao->crtol, &tao->crtol, &flg));
-  if (flg) tao->crtol_changed = PETSC_TRUE;
-  PetscCall(PetscOptionsReal("-tao_gatol", "Stop if norm of gradient less than", "TaoSetTolerances", tao->gatol, &tao->gatol, &flg));
-  if (flg) tao->gatol_changed = PETSC_TRUE;
-  PetscCall(PetscOptionsReal("-tao_grtol", "Stop if norm of gradient divided by the function value is less than", "TaoSetTolerances", tao->grtol, &tao->grtol, &flg));
-  if (flg) tao->grtol_changed = PETSC_TRUE;
-  PetscCall(PetscOptionsReal("-tao_gttol", "Stop if the norm of the gradient is less than the norm of the initial gradient times tol", "TaoSetTolerances", tao->gttol, &tao->gttol, &flg));
-  if (flg) tao->gttol_changed = PETSC_TRUE;
+  catol = tao->catol;
+  crtol = tao->crtol;
+  PetscCall(PetscOptionsReal("-tao_catol", "Stop if constraints violations within", "TaoSetConstraintTolerances", tao->catol, &catol, NULL));
+  PetscCall(PetscOptionsReal("-tao_crtol", "Stop if relative constraint violations within", "TaoSetConstraintTolerances", tao->crtol, &crtol, NULL));
+  PetscCall(TaoSetConstraintTolerances(tao, catol, crtol));
+
+  gatol = tao->gatol;
+  grtol = tao->grtol;
+  gttol = tao->gttol;
+  PetscCall(PetscOptionsReal("-tao_gatol", "Stop if norm of gradient less than", "TaoSetTolerances", tao->gatol, &gatol, NULL));
+  PetscCall(PetscOptionsReal("-tao_grtol", "Stop if norm of gradient divided by the function value is less than", "TaoSetTolerances", tao->grtol, &grtol, NULL));
+  PetscCall(PetscOptionsReal("-tao_gttol", "Stop if the norm of the gradient is less than the norm of the initial gradient times tol", "TaoSetTolerances", tao->gttol, &gttol, NULL));
+  PetscCall(TaoSetTolerances(tao, gatol, grtol, gttol));
+
   PetscCall(PetscOptionsInt("-tao_max_it", "Stop if iteration number exceeds", "TaoSetMaximumIterations", tao->max_it, &tao->max_it, &flg));
-  if (flg) tao->max_it_changed = PETSC_TRUE;
+  if (flg) PetscCall(TaoSetMaximumIterations(tao, tao->max_it));
+
   PetscCall(PetscOptionsInt("-tao_max_funcs", "Stop if number of function evaluations exceeds", "TaoSetMaximumFunctionEvaluations", tao->max_funcs, &tao->max_funcs, &flg));
-  if (flg) tao->max_funcs_changed = PETSC_TRUE;
-  PetscCall(PetscOptionsReal("-tao_fmin", "Stop if function less than", "TaoSetFunctionLowerBound", tao->fmin, &tao->fmin, &flg));
-  if (flg) tao->fmin_changed = PETSC_TRUE;
-  PetscCall(PetscOptionsReal("-tao_steptol", "Stop if step size or trust region radius less than", "", tao->steptol, &tao->steptol, &flg));
-  if (flg) tao->steptol_changed = PETSC_TRUE;
-  PetscCall(PetscOptionsReal("-tao_trust0", "Initial trust region radius", "TaoSetTrustRegionRadius", tao->trust0, &tao->trust0, &flg));
-  if (flg) tao->trust0_changed = PETSC_TRUE;
+  if (flg) PetscCall(TaoSetMaximumFunctionEvaluations(tao, tao->max_funcs));
+
+  PetscCall(PetscOptionsReal("-tao_fmin", "Stop if function less than", "TaoSetFunctionLowerBound", tao->fmin, &tao->fmin, NULL));
+  PetscCall(PetscOptionsBoundedReal("-tao_steptol", "Stop if step size or trust region radius less than", "", tao->steptol, &tao->steptol, NULL, 0));
+  PetscCall(PetscOptionsReal("-tao_trust0", "Initial trust region radius", "TaoSetInitialTrustRegionRadius", tao->trust0, &tao->trust0, &flg));
+  if (flg) PetscCall(TaoSetInitialTrustRegionRadius(tao, tao->trust0));
 
   PetscCall(PetscOptionsDeprecated("-tao_solution_monitor", "-tao_monitor_solution", "3.21", NULL));
   PetscCall(PetscOptionsDeprecated("-tao_gradient_monitor", "-tao_monitor_gradient", "3.21", NULL));
@@ -429,50 +446,50 @@ PetscErrorCode TaoSetFromOptions(Tao tao)
   PetscCall(PetscOptionsString("-tao_monitor_solution", "View solution vector after each iteration", "TaoMonitorSet", "stdout", monfilename, sizeof(monfilename), &flg));
   if (flg) {
     PetscCall(PetscViewerASCIIOpen(comm, monfilename, &monviewer));
-    PetscCall(TaoMonitorSet(tao, TaoMonitorSolution, monviewer, (PetscErrorCode(*)(void **))PetscViewerDestroy));
+    PetscCall(TaoMonitorSet(tao, TaoMonitorSolution, monviewer, (PetscErrorCode (*)(void **))PetscViewerDestroy));
   }
 
   PetscCall(PetscOptionsBool("-tao_converged_reason", "Print reason for Tao converged", "TaoSolve", tao->printreason, &tao->printreason, NULL));
   PetscCall(PetscOptionsString("-tao_monitor_gradient", "View gradient vector for each iteration", "TaoMonitorSet", "stdout", monfilename, sizeof(monfilename), &flg));
   if (flg) {
     PetscCall(PetscViewerASCIIOpen(comm, monfilename, &monviewer));
-    PetscCall(TaoMonitorSet(tao, TaoMonitorGradient, monviewer, (PetscErrorCode(*)(void **))PetscViewerDestroy));
+    PetscCall(TaoMonitorSet(tao, TaoMonitorGradient, monviewer, (PetscErrorCode (*)(void **))PetscViewerDestroy));
   }
 
   PetscCall(PetscOptionsString("-tao_monitor_step", "View step vector after each iteration", "TaoMonitorSet", "stdout", monfilename, sizeof(monfilename), &flg));
   if (flg) {
     PetscCall(PetscViewerASCIIOpen(comm, monfilename, &monviewer));
-    PetscCall(TaoMonitorSet(tao, TaoMonitorStep, monviewer, (PetscErrorCode(*)(void **))PetscViewerDestroy));
+    PetscCall(TaoMonitorSet(tao, TaoMonitorStep, monviewer, (PetscErrorCode (*)(void **))PetscViewerDestroy));
   }
 
   PetscCall(PetscOptionsString("-tao_monitor_residual", "View least-squares residual vector after each iteration", "TaoMonitorSet", "stdout", monfilename, sizeof(monfilename), &flg));
   if (flg) {
     PetscCall(PetscViewerASCIIOpen(comm, monfilename, &monviewer));
-    PetscCall(TaoMonitorSet(tao, TaoMonitorResidual, monviewer, (PetscErrorCode(*)(void **))PetscViewerDestroy));
+    PetscCall(TaoMonitorSet(tao, TaoMonitorResidual, monviewer, (PetscErrorCode (*)(void **))PetscViewerDestroy));
   }
 
   PetscCall(PetscOptionsString("-tao_monitor", "Use the default convergence monitor", "TaoMonitorSet", "stdout", monfilename, sizeof(monfilename), &flg));
   if (flg) {
     PetscCall(PetscViewerASCIIOpen(comm, monfilename, &monviewer));
-    PetscCall(TaoMonitorSet(tao, TaoMonitorDefault, monviewer, (PetscErrorCode(*)(void **))PetscViewerDestroy));
+    PetscCall(TaoMonitorSet(tao, TaoMonitorDefault, monviewer, (PetscErrorCode (*)(void **))PetscViewerDestroy));
   }
 
   PetscCall(PetscOptionsString("-tao_monitor_globalization", "Use the convergence monitor with extra globalization info", "TaoMonitorSet", "stdout", monfilename, sizeof(monfilename), &flg));
   if (flg) {
     PetscCall(PetscViewerASCIIOpen(comm, monfilename, &monviewer));
-    PetscCall(TaoMonitorSet(tao, TaoMonitorGlobalization, monviewer, (PetscErrorCode(*)(void **))PetscViewerDestroy));
+    PetscCall(TaoMonitorSet(tao, TaoMonitorGlobalization, monviewer, (PetscErrorCode (*)(void **))PetscViewerDestroy));
   }
 
   PetscCall(PetscOptionsString("-tao_monitor_short", "Use the short convergence monitor", "TaoMonitorSet", "stdout", monfilename, sizeof(monfilename), &flg));
   if (flg) {
     PetscCall(PetscViewerASCIIOpen(comm, monfilename, &monviewer));
-    PetscCall(TaoMonitorSet(tao, TaoMonitorDefaultShort, monviewer, (PetscErrorCode(*)(void **))PetscViewerDestroy));
+    PetscCall(TaoMonitorSet(tao, TaoMonitorDefaultShort, monviewer, (PetscErrorCode (*)(void **))PetscViewerDestroy));
   }
 
   PetscCall(PetscOptionsString("-tao_monitor_constraint_norm", "Use the default convergence monitor with constraint norm", "TaoMonitorSet", "stdout", monfilename, sizeof(monfilename), &flg));
   if (flg) {
     PetscCall(PetscViewerASCIIOpen(comm, monfilename, &monviewer));
-    PetscCall(TaoMonitorSet(tao, TaoMonitorConstraintNorm, monviewer, (PetscErrorCode(*)(void **))PetscViewerDestroy));
+    PetscCall(TaoMonitorSet(tao, TaoMonitorConstraintNorm, monviewer, (PetscErrorCode (*)(void **))PetscViewerDestroy));
   }
 
   flg = PETSC_FALSE;
@@ -486,7 +503,7 @@ PetscErrorCode TaoSetFromOptions(Tao tao)
     TaoMonitorDrawCtx drawctx;
     PetscInt          howoften = 1;
     PetscCall(TaoMonitorDrawCtxCreate(PetscObjectComm((PetscObject)tao), NULL, NULL, PETSC_DECIDE, PETSC_DECIDE, 300, 300, howoften, &drawctx));
-    PetscCall(TaoMonitorSet(tao, TaoMonitorSolutionDraw, drawctx, (PetscErrorCode(*)(void **))TaoMonitorDrawCtxDestroy));
+    PetscCall(TaoMonitorSet(tao, TaoMonitorSolutionDraw, drawctx, (PetscErrorCode (*)(void **))TaoMonitorDrawCtxDestroy));
   }
 
   flg = PETSC_FALSE;
@@ -499,7 +516,7 @@ PetscErrorCode TaoSetFromOptions(Tao tao)
     TaoMonitorDrawCtx drawctx;
     PetscInt          howoften = 1;
     PetscCall(TaoMonitorDrawCtxCreate(PetscObjectComm((PetscObject)tao), NULL, NULL, PETSC_DECIDE, PETSC_DECIDE, 300, 300, howoften, &drawctx));
-    PetscCall(TaoMonitorSet(tao, TaoMonitorGradientDraw, drawctx, (PetscErrorCode(*)(void **))TaoMonitorDrawCtxDestroy));
+    PetscCall(TaoMonitorSet(tao, TaoMonitorGradientDraw, drawctx, (PetscErrorCode (*)(void **))TaoMonitorDrawCtxDestroy));
   }
   flg = PETSC_FALSE;
   PetscCall(PetscOptionsBool("-tao_fd_gradient", "compute gradient using finite differences", "TaoDefaultComputeGradient", flg, &flg, NULL));
@@ -542,7 +559,7 @@ PetscErrorCode TaoSetFromOptions(Tao tao)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TaoViewFromOptions - View a `Tao` object based on values in the options database
 
   Collective
@@ -564,7 +581,7 @@ PetscErrorCode TaoViewFromOptions(Tao A, PetscObject obj, const char name[])
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TaoView - Prints information about the `Tao` object
 
   Collective
@@ -604,28 +621,18 @@ PetscErrorCode TaoView(Tao tao, PetscViewer viewer)
   if (isascii) {
     PetscCall(PetscObjectPrintClassNamePrefixType((PetscObject)tao, viewer));
 
-    if (tao->ops->view) {
-      PetscCall(PetscViewerASCIIPushTab(viewer));
-      PetscUseTypeMethod(tao, view, viewer);
-      PetscCall(PetscViewerASCIIPopTab(viewer));
-    }
-    if (tao->linesearch) {
-      PetscCall(PetscViewerASCIIPushTab(viewer));
-      PetscCall(TaoLineSearchView(tao->linesearch, viewer));
-      PetscCall(PetscViewerASCIIPopTab(viewer));
-    }
+    PetscCall(PetscViewerASCIIPushTab(viewer));
+    PetscTryTypeMethod(tao, view, viewer);
+    if (tao->linesearch) PetscCall(TaoLineSearchView(tao->linesearch, viewer));
     if (tao->ksp) {
-      PetscCall(PetscViewerASCIIPushTab(viewer));
       PetscCall(KSPView(tao->ksp, viewer));
       PetscCall(PetscViewerASCIIPrintf(viewer, "total KSP iterations: %" PetscInt_FMT "\n", tao->ksp_tot_its));
-      PetscCall(PetscViewerASCIIPopTab(viewer));
     }
 
-    PetscCall(PetscViewerASCIIPushTab(viewer));
-
     if (tao->XL || tao->XU) PetscCall(PetscViewerASCIIPrintf(viewer, "Active Set subset type: %s\n", TaoSubSetTypes[tao->subset_type]));
 
     PetscCall(PetscViewerASCIIPrintf(viewer, "convergence tolerances: gatol=%g,", (double)tao->gatol));
+    PetscCall(PetscViewerASCIIPrintf(viewer, " grtol=%g,", (double)tao->grtol));
     PetscCall(PetscViewerASCIIPrintf(viewer, " steptol=%g,", (double)tao->steptol));
     PetscCall(PetscViewerASCIIPrintf(viewer, " gttol=%g\n", (double)tao->gttol));
     PetscCall(PetscViewerASCIIPrintf(viewer, "Residual in Function/Gradient:=%g\n", (double)tao->residual));
@@ -650,15 +657,18 @@ PetscErrorCode TaoView(Tao tao, PetscViewer viewer)
 
     if (tao->nfuncs > 0) {
       PetscCall(PetscViewerASCIIPrintf(viewer, "total number of function evaluations=%" PetscInt_FMT ",", tao->nfuncs));
-      PetscCall(PetscViewerASCIIPrintf(viewer, "                max: %" PetscInt_FMT "\n", tao->max_funcs));
+      if (tao->max_funcs == PETSC_UNLIMITED) PetscCall(PetscViewerASCIIPrintf(viewer, "                (max: unlimited)\n"));
+      else PetscCall(PetscViewerASCIIPrintf(viewer, "                (max: %" PetscInt_FMT ")\n", tao->max_funcs));
     }
     if (tao->ngrads > 0) {
       PetscCall(PetscViewerASCIIPrintf(viewer, "total number of gradient evaluations=%" PetscInt_FMT ",", tao->ngrads));
-      PetscCall(PetscViewerASCIIPrintf(viewer, "                max: %" PetscInt_FMT "\n", tao->max_funcs));
+      if (tao->max_funcs == PETSC_UNLIMITED) PetscCall(PetscViewerASCIIPrintf(viewer, "                (max: unlimited)\n"));
+      else PetscCall(PetscViewerASCIIPrintf(viewer, "                (max: %" PetscInt_FMT ")\n", tao->max_funcs));
     }
     if (tao->nfuncgrads > 0) {
       PetscCall(PetscViewerASCIIPrintf(viewer, "total number of function/gradient evaluations=%" PetscInt_FMT ",", tao->nfuncgrads));
-      PetscCall(PetscViewerASCIIPrintf(viewer, "    (max: %" PetscInt_FMT ")\n", tao->max_funcs));
+      if (tao->max_funcs == PETSC_UNLIMITED) PetscCall(PetscViewerASCIIPrintf(viewer, "    (max: unlimited)\n"));
+      else PetscCall(PetscViewerASCIIPrintf(viewer, "    (max: %" PetscInt_FMT ")\n", tao->max_funcs));
     }
     if (tao->nhess > 0) PetscCall(PetscViewerASCIIPrintf(viewer, "total number of Hessian evaluations=%" PetscInt_FMT "\n", tao->nhess));
     if (tao->nconstraints > 0) PetscCall(PetscViewerASCIIPrintf(viewer, "total number of constraint function evaluations=%" PetscInt_FMT "\n", tao->nconstraints));
@@ -686,11 +696,13 @@ PetscErrorCode TaoView(Tao tao, PetscViewer viewer)
         PetscCall(PetscViewerASCIIPrintf(viewer, " User Terminated\n"));
         break;
       default:
-        PetscCall(PetscViewerASCIIPrintf(viewer, "\n"));
+        PetscCall(PetscViewerASCIIPrintf(viewer, " %d\n", tao->reason));
         break;
       }
+    } else if (tao->reason == TAO_CONTINUE_ITERATING) {
+      PetscCall(PetscViewerASCIIPrintf(viewer, "Solver never run\n"));
     } else {
-      PetscCall(PetscViewerASCIIPrintf(viewer, "Solver terminated: %d", tao->reason));
+      PetscCall(PetscViewerASCIIPrintf(viewer, "Solver failed: "));
       switch (tao->reason) {
       case TAO_DIVERGED_MAXITS:
         PetscCall(PetscViewerASCIIPrintf(viewer, " Maximum Iterations\n"));
@@ -711,7 +723,7 @@ PetscErrorCode TaoView(Tao tao, PetscViewer viewer)
         PetscCall(PetscViewerASCIIPrintf(viewer, " User Terminated\n"));
         break;
       default:
-        PetscCall(PetscViewerASCIIPrintf(viewer, "\n"));
+        PetscCall(PetscViewerASCIIPrintf(viewer, " %d\n", tao->reason));
         break;
       }
     }
@@ -813,8 +825,13 @@ PetscErrorCode TaoGetRecycleHistory(Tao tao, PetscBool *recycle)
 
   Level: beginner
 
-  Note:
-  Use `PETSC_DEFAULT` to leave one or more tolerances unchanged.
+  Notes:
+  Use `PETSC_CURRENT` to leave one or more tolerances unchanged.
+
+  Use `PETSC_DETERMINE` to set one or more tolerances to their values when the `tao`object's type was set
+
+  Fortran Note:
+  Use `PETSC_CURRENT_REAL` or `PETSC_DETERMINE_REAL`
 
 .seealso: [](ch_tao), `Tao`, `TaoConvergedReason`, `TaoGetTolerances()`
 @*/
@@ -826,31 +843,25 @@ PetscErrorCode TaoSetTolerances(Tao tao, PetscReal gatol, PetscReal grtol, Petsc
   PetscValidLogicalCollectiveReal(tao, grtol, 3);
   PetscValidLogicalCollectiveReal(tao, gttol, 4);
 
-  if (gatol != (PetscReal)PETSC_DEFAULT) {
-    if (gatol < 0) {
-      PetscCall(PetscInfo(tao, "Tried to set negative gatol -- ignored.\n"));
-    } else {
-      tao->gatol         = PetscMax(0, gatol);
-      tao->gatol_changed = PETSC_TRUE;
-    }
+  if (gatol == (PetscReal)PETSC_DETERMINE) {
+    tao->gatol = tao->default_gatol;
+  } else if (gatol != (PetscReal)PETSC_CURRENT) {
+    PetscCheck(gatol >= 0, PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_OUTOFRANGE, "Negative gatol not allowed");
+    tao->gatol = gatol;
   }
 
-  if (grtol != (PetscReal)PETSC_DEFAULT) {
-    if (grtol < 0) {
-      PetscCall(PetscInfo(tao, "Tried to set negative grtol -- ignored.\n"));
-    } else {
-      tao->grtol         = PetscMax(0, grtol);
-      tao->grtol_changed = PETSC_TRUE;
-    }
+  if (grtol == (PetscReal)PETSC_DETERMINE) {
+    tao->grtol = tao->default_grtol;
+  } else if (grtol != (PetscReal)PETSC_CURRENT) {
+    PetscCheck(grtol >= 0, PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_OUTOFRANGE, "Negative grtol not allowed");
+    tao->grtol = grtol;
   }
 
-  if (gttol != (PetscReal)PETSC_DEFAULT) {
-    if (gttol < 0) {
-      PetscCall(PetscInfo(tao, "Tried to set negative gttol -- ignored.\n"));
-    } else {
-      tao->gttol         = PetscMax(0, gttol);
-      tao->gttol_changed = PETSC_TRUE;
-    }
+  if (gttol == (PetscReal)PETSC_DETERMINE) {
+    tao->gttol = tao->default_gttol;
+  } else if (gttol != (PetscReal)PETSC_CURRENT) {
+    PetscCheck(gttol >= 0, PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_OUTOFRANGE, "Negative gttol not allowed");
+    tao->gttol = gttol;
   }
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -862,8 +873,8 @@ PetscErrorCode TaoSetTolerances(Tao tao, PetscReal gatol, PetscReal grtol, Petsc
 
   Input Parameters:
 + tao   - the `Tao` context
-. catol - absolute constraint tolerance, constraint norm must be less than `catol` for used for gatol convergence criteria
-- crtol - relative constraint tolerance, constraint norm must be less than `crtol` for used for gatol, gttol convergence criteria
+. catol - absolute constraint tolerance, constraint norm must be less than `catol` for used for `gatol` convergence criteria
+- crtol - relative constraint tolerance, constraint norm must be less than `crtol` for used for `gatol`, `gttol` convergence criteria
 
   Options Database Keys:
 + -tao_catol <catol> - Sets catol
@@ -872,7 +883,12 @@ PetscErrorCode TaoSetTolerances(Tao tao, PetscReal gatol, PetscReal grtol, Petsc
   Level: intermediate
 
   Notes:
-  Use `PETSC_DEFAULT` to leave any tolerance unchanged.
+  Use `PETSC_CURRENT` to leave one or tolerance unchanged.
+
+  Use `PETSC_DETERMINE` to set one or more tolerances to their values when the `tao` object's type was set
+
+  Fortran Note:
+  Use `PETSC_CURRENT_REAL` or `PETSC_DETERMINE_REAL`
 
 .seealso: [](ch_tao), `Tao`, `TaoConvergedReason`, `TaoGetTolerances()`, `TaoGetConstraintTolerances()`, `TaoSetTolerances()`
 @*/
@@ -883,22 +899,18 @@ PetscErrorCode TaoSetConstraintTolerances(Tao tao, PetscReal catol, PetscReal cr
   PetscValidLogicalCollectiveReal(tao, catol, 2);
   PetscValidLogicalCollectiveReal(tao, crtol, 3);
 
-  if (catol != (PetscReal)PETSC_DEFAULT) {
-    if (catol < 0) {
-      PetscCall(PetscInfo(tao, "Tried to set negative catol -- ignored.\n"));
-    } else {
-      tao->catol         = PetscMax(0, catol);
-      tao->catol_changed = PETSC_TRUE;
-    }
+  if (catol == (PetscReal)PETSC_DETERMINE) {
+    tao->catol = tao->default_catol;
+  } else if (catol != (PetscReal)PETSC_CURRENT) {
+    PetscCheck(catol >= 0, PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_OUTOFRANGE, "Negative catol not allowed");
+    tao->catol = catol;
   }
 
-  if (crtol != (PetscReal)PETSC_DEFAULT) {
-    if (crtol < 0) {
-      PetscCall(PetscInfo(tao, "Tried to set negative crtol -- ignored.\n"));
-    } else {
-      tao->crtol         = PetscMax(0, crtol);
-      tao->crtol_changed = PETSC_TRUE;
-    }
+  if (crtol == (PetscReal)PETSC_DETERMINE) {
+    tao->crtol = tao->default_crtol;
+  } else if (crtol != (PetscReal)PETSC_CURRENT) {
+    PetscCheck(crtol >= 0, PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_OUTOFRANGE, "Negative crtol not allowed");
+    tao->crtol = crtol;
   }
   PetscFunctionReturn(PETSC_SUCCESS);
 }
@@ -912,8 +924,8 @@ PetscErrorCode TaoSetConstraintTolerances(Tao tao, PetscReal catol, PetscReal cr
 . tao - the `Tao` context
 
   Output Parameters:
-+ catol - absolute constraint tolerance, constraint norm must be less than `catol` for used for gatol convergence criteria
-- crtol - relative constraint tolerance, constraint norm must be less than `crtol` for used for gatol, gttol convergence criteria
++ catol - absolute constraint tolerance, constraint norm must be less than `catol` for used for `gatol` convergence criteria
+- crtol - relative constraint tolerance, constraint norm must be less than `crtol` for used for `gatol`, `gttol` convergence criteria
 
   Level: intermediate
 
@@ -951,8 +963,7 @@ PetscErrorCode TaoSetFunctionLowerBound(Tao tao, PetscReal fmin)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(tao, TAO_CLASSID, 1);
   PetscValidLogicalCollectiveReal(tao, fmin, 2);
-  tao->fmin         = fmin;
-  tao->fmin_changed = PETSC_TRUE;
+  tao->fmin = fmin;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -989,13 +1000,19 @@ PetscErrorCode TaoGetFunctionLowerBound(Tao tao, PetscReal *fmin)
 
   Input Parameters:
 + tao  - the `Tao` solver context
-- nfcn - the maximum number of function evaluations (>=0)
+- nfcn - the maximum number of function evaluations (>=0), use `PETSC_UNLIMITED` to have no bound
 
   Options Database Key:
 . -tao_max_funcs <nfcn> - sets the maximum number of function evaluations
 
   Level: intermediate
 
+  Note:
+  Use `PETSC_DETERMINE` to use the default maximum number of function evaluations that was set when the object type was set.
+
+  Developer Note:
+  Deprecated support for an unlimited number of function evaluations by passing a negative value.
+
 .seealso: [](ch_tao), `Tao`, `TaoSetTolerances()`, `TaoSetMaximumIterations()`
 @*/
 PetscErrorCode TaoSetMaximumFunctionEvaluations(Tao tao, PetscInt nfcn)
@@ -1003,12 +1020,14 @@ PetscErrorCode TaoSetMaximumFunctionEvaluations(Tao tao, PetscInt nfcn)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(tao, TAO_CLASSID, 1);
   PetscValidLogicalCollectiveInt(tao, nfcn, 2);
-  if (nfcn >= 0) {
-    tao->max_funcs = PetscMax(0, nfcn);
+  if (nfcn == PETSC_DETERMINE) {
+    tao->max_funcs = tao->default_max_funcs;
+  } else if (nfcn == PETSC_UNLIMITED || nfcn < 0) {
+    tao->max_funcs = PETSC_UNLIMITED;
   } else {
-    tao->max_funcs = -1;
+    PetscCheck(nfcn >= 0, PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_OUTOFRANGE, "Maxium number of function evaluations  must be positive");
+    tao->max_funcs = nfcn;
   }
-  tao->max_funcs_changed = PETSC_TRUE;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -1067,13 +1086,19 @@ PetscErrorCode TaoGetCurrentFunctionEvaluations(Tao tao, PetscInt *nfuncs)
 
   Input Parameters:
 + tao    - the `Tao` solver context
-- maxits - the maximum number of iterates (>=0)
+- maxits - the maximum number of iterates (>=0), use `PETSC_UNLIMITED` to have no bound
 
   Options Database Key:
 . -tao_max_it <its> - sets the maximum number of iterations
 
   Level: intermediate
 
+  Note:
+  Use `PETSC_DETERMINE` to use the default maximum number of iterations that was set when the object's type was set.
+
+  Developer Note:
+  DeprAlso accepts the deprecated negative values to indicate no limit
+
 .seealso: [](ch_tao), `Tao`, `TaoSetTolerances()`, `TaoSetMaximumFunctionEvaluations()`
 @*/
 PetscErrorCode TaoSetMaximumIterations(Tao tao, PetscInt maxits)
@@ -1081,8 +1106,14 @@ PetscErrorCode TaoSetMaximumIterations(Tao tao, PetscInt maxits)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(tao, TAO_CLASSID, 1);
   PetscValidLogicalCollectiveInt(tao, maxits, 2);
-  tao->max_it         = PetscMax(0, maxits);
-  tao->max_it_changed = PETSC_TRUE;
+  if (maxits == PETSC_DETERMINE) {
+    tao->max_it = tao->default_max_it;
+  } else if (maxits == PETSC_UNLIMITED) {
+    tao->max_it = PETSC_INT_MAX;
+  } else {
+    PetscCheck(maxits > 0, PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_OUTOFRANGE, "Maxium number of iterations must be positive");
+    tao->max_it = maxits;
+  }
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -1124,6 +1155,9 @@ PetscErrorCode TaoGetMaximumIterations(Tao tao, PetscInt *maxits)
 
   Level: intermediate
 
+  Note:
+  Use `PETSC_DETERMINE` to use the default radius that was set when the object's type was set.
+
 .seealso: [](ch_tao), `Tao`, `TaoGetTrustRegionRadius()`, `TaoSetTrustRegionTolerance()`, `TAONTR`
 @*/
 PetscErrorCode TaoSetInitialTrustRegionRadius(Tao tao, PetscReal radius)
@@ -1131,8 +1165,12 @@ PetscErrorCode TaoSetInitialTrustRegionRadius(Tao tao, PetscReal radius)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(tao, TAO_CLASSID, 1);
   PetscValidLogicalCollectiveReal(tao, radius, 2);
-  tao->trust0         = PetscMax(0.0, radius);
-  tao->trust0_changed = PETSC_TRUE;
+  if (radius == PETSC_DETERMINE) {
+    tao->trust0 = tao->default_trust0;
+  } else {
+    PetscCheck(radius > 0, PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_OUTOFRANGE, "Radius must be positive");
+    tao->trust0 = radius;
+  }
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -1391,18 +1429,24 @@ PetscErrorCode TaoResetStatistics(Tao tao)
   Logically Collective
 
   Input Parameters:
-+ tao  - The `Tao` solver context
-- func - The function
++ tao  - The `Tao` solver
+. func - The function
+- ctx  - The update function context
 
   Calling sequence of `func`:
-+ tao - the optimizer context
-- ctx - The current step of the iteration
++ tao - The optimizer context
+. it  - The current iteration index
+- ctx - The update context
 
   Level: advanced
 
+  Notes:
+  Users can modify the gradient direction or any other vector associated to the specific solver used.
+  The objective function value is always recomputed after a call to the update hook.
+
 .seealso: [](ch_tao), `Tao`, `TaoSolve()`
 @*/
-PetscErrorCode TaoSetUpdate(Tao tao, PetscErrorCode (*func)(Tao, PetscInt, void *), void *ctx)
+PetscErrorCode TaoSetUpdate(Tao tao, PetscErrorCode (*func)(Tao tao, PetscInt it, void *ctx), void *ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(tao, TAO_CLASSID, 1);
@@ -1413,7 +1457,7 @@ PetscErrorCode TaoSetUpdate(Tao tao, PetscErrorCode (*func)(Tao, PetscInt, void
 
 /*@C
   TaoSetConvergenceTest - Sets the function that is to be used to test
-  for convergence o fthe iterative minimization solution.  The new convergence
+  for convergence of the iterative minimization solution.  The new convergence
   testing routine will replace Tao's default convergence test.
 
   Logically Collective
@@ -1488,7 +1532,7 @@ PetscErrorCode TaoMonitorSet(Tao tao, PetscErrorCode (*func)(Tao, void *), void
   PetscCheck(tao->numbermonitors < MAXTAOMONITORS, PetscObjectComm((PetscObject)tao), PETSC_ERR_SUP, "Cannot attach another monitor -- max=%d", MAXTAOMONITORS);
 
   for (i = 0; i < tao->numbermonitors; i++) {
-    PetscCall(PetscMonitorCompare((PetscErrorCode(*)(void))func, ctx, dest, (PetscErrorCode(*)(void))tao->monitor[i], tao->monitorcontext[i], tao->monitordestroy[i], &identical));
+    PetscCall(PetscMonitorCompare((PetscErrorCode (*)(void))func, ctx, dest, (PetscErrorCode (*)(void))tao->monitor[i], tao->monitorcontext[i], tao->monitordestroy[i], &identical));
     if (identical) PetscFunctionReturn(PETSC_SUCCESS);
   }
   tao->monitor[tao->numbermonitors]        = func;
@@ -1819,7 +1863,12 @@ PetscErrorCode TaoMonitorStep(Tao tao, void *ctx)
 
   Level: advanced
 
-.seealso: [](ch_tao), `Tao`, `TaoMonitorSolution()`, `TaoMonitorSet()`, `TaoMonitorGradientDraw()`
+  Note:
+  The context created by `TaoMonitorDrawCtxCreate()`, along with `TaoMonitorSolutionDraw()`, and `TaoMonitorDrawCtxDestroy()`
+  are passed to `TaoMonitorSet()` to monitor the solution graphically.
+
+.seealso: [](ch_tao), `Tao`, `TaoMonitorSolution()`, `TaoMonitorSet()`, `TaoMonitorGradientDraw()`, `TaoMonitorDrawCtxCreate()`,
+          `TaoMonitorDrawCtxDestroy()`
 @*/
 PetscErrorCode TaoMonitorSolutionDraw(Tao tao, void *ctx)
 {
@@ -1963,7 +2012,7 @@ PetscErrorCode TaoDefaultConvergenceTest(Tao tao, void *dummy)
   } else if (gnorm0 != 0 && ((gttol == 0 && gnorm == 0) || gnorm / gnorm0 < gttol) && cnorm <= crtol) {
     PetscCall(PetscInfo(tao, "Converged due to relative residual norm ||g(X)||/||g(X0)|| = %g < %g\n", (double)(gnorm / gnorm0), (double)gttol));
     reason = TAO_CONVERGED_GTTOL;
-  } else if (max_funcs >= 0 && nfuncs > max_funcs) {
+  } else if (max_funcs != PETSC_UNLIMITED && nfuncs > max_funcs) {
     PetscCall(PetscInfo(tao, "Exceeded maximum number of function evaluations: %" PetscInt_FMT " > %" PetscInt_FMT "\n", nfuncs, max_funcs));
     reason = TAO_DIVERGED_MAXFCN;
   } else if (tao->lsflag != 0) {
@@ -1982,7 +2031,7 @@ PetscErrorCode TaoDefaultConvergenceTest(Tao tao, void *dummy)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TaoSetOptionsPrefix - Sets the prefix used for searching for all
   Tao options in the database.
 
@@ -2023,7 +2072,7 @@ PetscErrorCode TaoSetOptionsPrefix(Tao tao, const char p[])
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TaoAppendOptionsPrefix - Appends to the prefix used for searching for all Tao options in the database.
 
   Logically Collective
@@ -2050,7 +2099,7 @@ PetscErrorCode TaoAppendOptionsPrefix(Tao tao, const char p[])
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TaoGetOptionsPrefix - Gets the prefix used for searching for all
   Tao options in the database
 
@@ -2077,7 +2126,7 @@ PetscErrorCode TaoGetOptionsPrefix(Tao tao, const char *p[])
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TaoSetType - Sets the `TaoType` for the minimization solver.
 
   Collective
@@ -2105,7 +2154,7 @@ PetscErrorCode TaoSetType(Tao tao, TaoType type)
   PetscCall(PetscObjectTypeCompare((PetscObject)tao, type, &issame));
   if (issame) PetscFunctionReturn(PETSC_SUCCESS);
 
-  PetscCall(PetscFunctionListFind(TaoList, type, (void (**)(void)) & create_xxx));
+  PetscCall(PetscFunctionListFind(TaoList, type, (void (**)(void))&create_xxx));
   PetscCheck(create_xxx, PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_UNKNOWN_TYPE, "Unable to find requested Tao type %s", type);
 
   /* Destroy the existing solver information */
@@ -2128,7 +2177,7 @@ PetscErrorCode TaoSetType(Tao tao, TaoType type)
 /*@C
   TaoRegister - Adds a method to the Tao package for minimization.
 
-  Not Collective
+  Not Collective, No Fortran Support
 
   Input Parameters:
 + sname - name of a new user-defined solver
@@ -2397,7 +2446,7 @@ PetscErrorCode TaoGetSolutionStatus(Tao tao, PetscInt *its, PetscReal *f, PetscR
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TaoGetType - Gets the current `TaoType` being used in the `Tao` object
 
   Not Collective
@@ -2482,8 +2531,7 @@ PetscErrorCode TaoMonitor(Tao tao, PetscInt its, PetscReal f, PetscReal res, Pet
   Notes:
   If set, `Tao` will fill the given arrays with the indicated
   information at each iteration.  If 'obj','resid','cnorm','lits' are
-  *all* `NULL` then space (using size `na`, or 1000 if na is `PETSC_DECIDE` or
-  `PETSC_DEFAULT`) is allocated for the history.
+  *all* `NULL` then space (using size `na`, or 1000 if `na` is `PETSC_DECIDE`) is allocated for the history.
   If not all are `NULL`, then only the non-`NULL` information categories
   will be stored, the others will be ignored.
 
@@ -2504,7 +2552,7 @@ PetscErrorCode TaoSetConvergenceHistory(Tao tao, PetscReal obj[], PetscReal resi
   if (cnorm) PetscAssertPointer(cnorm, 4);
   if (lits) PetscAssertPointer(lits, 5);
 
-  if (na == PETSC_DECIDE || na == PETSC_DEFAULT) na = 1000;
+  if (na == PETSC_DECIDE || na == PETSC_CURRENT) na = 1000;
   if (!obj && !resid && !cnorm && !lits) {
     PetscCall(PetscCalloc4(na, &obj, na, &resid, na, &cnorm, na, &lits));
     tao->hist_malloc = PETSC_TRUE;
@@ -2612,7 +2660,7 @@ PetscErrorCode TaoGetApplicationContext(Tao tao, void *usrP)
 }
 
 /*@
-  TaoSetGradientNorm - Sets the matrix used to define the norm that measures the size of the gradient.
+  TaoSetGradientNorm - Sets the matrix used to define the norm that measures the size of the gradient in some of the `Tao` algorithms
 
   Collective
 
@@ -2638,7 +2686,7 @@ PetscErrorCode TaoSetGradientNorm(Tao tao, Mat M)
 }
 
 /*@
-  TaoGetGradientNorm - Returns the matrix used to define the norm used for measuring the size of the gradient.
+  TaoGetGradientNorm - Returns the matrix used to define the norm used for measuring the size of the gradient in some of the `Tao` algorithms
 
   Not Collective
 
@@ -2661,14 +2709,14 @@ PetscErrorCode TaoGetGradientNorm(Tao tao, Mat *M)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TaoGradientNorm - Compute the norm using the `NormType`, the user has selected
 
   Collective
 
   Input Parameters:
 + tao      - the `Tao` context
-. gradient - the gradient to be computed
+. gradient - the gradient
 - type     - the norm type
 
   Output Parameter:
@@ -2676,6 +2724,15 @@ PetscErrorCode TaoGetGradientNorm(Tao tao, Mat *M)
 
   Level: advanced
 
+  Note:
+  If `TaoSetGradientNorm()` has been set and `type` is `NORM_2` then the norm provided with `TaoSetGradientNorm()` is used.
+
+  Developer Notes:
+  Should be named `TaoComputeGradientNorm()`.
+
+  The usage is a bit confusing, with `TaoSetGradientNorm()` plus `NORM_2` resulting in the computation of the user provided
+  norm, perhaps a refactorization is in order.
+
 .seealso: [](ch_tao), `Tao`, `TaoSetGradientNorm()`, `TaoGetGradientNorm()`
 @*/
 PetscErrorCode TaoGradientNorm(Tao tao, Vec gradient, NormType type, PetscReal *gnorm)
@@ -2699,7 +2756,7 @@ PetscErrorCode TaoGradientNorm(Tao tao, Vec gradient, NormType type, PetscReal *
 }
 
 /*@C
-  TaoMonitorDrawCtxCreate - Creates the monitor context for `TaoMonitorDrawSolution()`
+  TaoMonitorDrawCtxCreate - Creates the monitor context for `TaoMonitorSolutionDraw()`
 
   Collective
 
@@ -2716,11 +2773,16 @@ PetscErrorCode TaoGradientNorm(Tao tao, Vec gradient, NormType type, PetscReal *
   Output Parameter:
 . ctx - the monitor context
 
-  Options Database Key:
-. -tao_draw_solution_initial - show initial guess as well as current solution
+  Options Database Keys:
++ -tao_monitor_solution_draw - use `TaoMonitorSolutionDraw()` to monitor the solution
+- -tao_draw_solution_initial - show initial guess as well as current solution
 
   Level: intermediate
 
+  Note:
+  The context this creates, along with `TaoMonitorSolutionDraw()`, and `TaoMonitorDrawCtxDestroy()`
+  are passed to `TaoMonitorSet()`.
+
 .seealso: [](ch_tao), `Tao`, `TaoMonitorSet()`, `TaoMonitorDefault()`, `VecView()`, `TaoMonitorDrawCtx()`
 @*/
 PetscErrorCode TaoMonitorDrawCtxCreate(MPI_Comm comm, const char host[], const char label[], int x, int y, int m, int n, PetscInt howoften, TaoMonitorDrawCtx *ctx)
@@ -2734,7 +2796,7 @@ PetscErrorCode TaoMonitorDrawCtxCreate(MPI_Comm comm, const char host[], const c
 }
 
 /*@C
-  TaoMonitorDrawCtxDestroy - Destroys the monitor context for `TaoMonitorDrawSolution()`
+  TaoMonitorDrawCtxDestroy - Destroys the monitor context for `TaoMonitorSolutionDraw()`
 
   Collective
 
@@ -2743,7 +2805,11 @@ PetscErrorCode TaoMonitorDrawCtxCreate(MPI_Comm comm, const char host[], const c
 
   Level: intermediate
 
-.seealso: [](ch_tao), `Tao`, `TaoMonitorSet()`, `TaoMonitorDefault()`, `VecView()`, `TaoMonitorDrawSolution()`
+  Note:
+  This is passed to `TaoMonitorSet()` as the final argument, along with `TaoMonitorSolutionDraw()`, and the context
+  obtained with `TaoMonitorDrawCtxCreate()`.
+
+.seealso: [](ch_tao), `Tao`, `TaoMonitorSet()`, `TaoMonitorDefault()`, `VecView()`, `TaoMonitorSolutionDraw()`
 @*/
 PetscErrorCode TaoMonitorDrawCtxDestroy(TaoMonitorDrawCtx *ictx)
 {
